"use strict";
/* tslint:disable:max-classes-per-file */
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var R = require('ramda');
var fs = require("fs");
var path = require("path");
var mkdirp = require("mkdirp");
var eslint_1 = require("./eslint");
var factory_1 = require("./test-structure/factory");
var removeExtension = require('./path-helpers').removeExtension;
exports.getSettings = function (ide) {
    var pkgJson = ide.getPkgJson();
    var mergeSetting = function (settingName, defaultValue) {
        var userSetting = R.pathOr(ide.getSetting(settingName), ['fancyReact', settingName], pkgJson);
        return userSetting || defaultValue;
    };
    return {
        packagePath: mergeSetting("packagePath", "client"),
        pkgJson: pkgJson,
        projectRoot: ide.getProjectRoot(),
        sourcePath: mergeSetting("sourcePath", "src"),
        testPath: mergeSetting("testPath", "test"),
        testStructure: mergeSetting("testStructure", "ParallelDirs"),
        testSuffix: mergeSetting("testSuffix", "-test"),
    };
};
var getProjectPaths = function (settings) {
    return {
        projectRoot: settings.projectRoot,
        srcInsideProject: path.join('/' + settings.packagePath, settings.sourcePath),
        testInsideProject: path.join('/' + settings.packagePath, settings.testPath),
    };
};
var getProjectFuncs = function (projectPaths, settings) {
    var packagePath = settings.packagePath, projectRoot = settings.projectRoot, sourcePath = settings.sourcePath;
    var srcInsideProject = projectPaths.srcInsideProject;
    var componentDetails = function (componentName) {
        var folderPath = projectRoot + "/" + packagePath + "/" + sourcePath + "/components/" + componentName;
        var componentPath = folderPath + "/" + componentName + ".js";
        var stylesPath = folderPath + "/" + componentName + ".scss";
        return {
            componentName: componentName,
            componentPath: componentPath,
            folderPath: folderPath,
            projectRoot: projectRoot,
            stylesPath: stylesPath,
        };
    };
    var fullPathToProjectPath = function (sourceFile) {
        if (!sourceFile.startsWith(settings.projectRoot)) {
            throw new Error("Source file " + sourceFile + " not part of project " + settings.projectRoot);
        }
        return sourceFile.slice(settings.projectRoot.length);
    };
    var sourcePathWithinSrc = function (sourceFileWithinProject) {
        return sourceFileWithinProject.slice(srcInsideProject.length);
    };
    var sourceFileToModulePath = function (sourceFile) {
        var sourceFileWithinProject = fullPathToProjectPath(sourceFile);
        return 'src' + removeExtension(sourcePathWithinSrc(sourceFileWithinProject));
    };
    return {
        componentDetails: componentDetails,
        fullPathToProjectPath: fullPathToProjectPath,
        sourceFileToModulePath: sourceFileToModulePath,
        sourcePathWithinSrc: sourcePathWithinSrc,
    };
};
exports.getProject = function (settings) {
    var projectRoot = settings.projectRoot;
    var projectPaths = getProjectPaths(settings);
    var srcInsideProject = projectPaths.srcInsideProject, testInsideProject = projectPaths.testInsideProject;
    var projectFuncs = getProjectFuncs(projectPaths, settings);
    var componentDetails = projectFuncs.componentDetails, fullPathToProjectPath = projectFuncs.fullPathToProjectPath, sourceFileToModulePath = projectFuncs.sourceFileToModulePath, sourcePathWithinSrc = projectFuncs.sourcePathWithinSrc;
    var testFuncs = factory_1.getTestFuncs(projectFuncs, projectPaths, settings);
    var sourceFileWPToTestFileWP = testFuncs.sourceFileWPToTestFileWP, isPathWPTestFile = testFuncs.isPathWPTestFile, testFileWPToSourceFileWP = testFuncs.testFileWPToSourceFileWP;
    var isPathTestFile = function (filePath) {
        var filePathWithinProject = filePath.startsWith(projectRoot) ?
            filePath.slice(projectRoot.length) :
            filePath;
        return isPathWPTestFile(filePathWithinProject);
    };
    var testFileToSourceFile = function (testFile) {
        var testFileWithinProject = fullPathToProjectPath(testFile);
        return path.join(projectRoot, testFileWPToSourceFileWP(testFileWithinProject));
    };
    var sourceFileToTestFile = function (sourceFile) {
        var sourceFileWithinProject = fullPathToProjectPath(sourceFile);
        if (!sourceFileWithinProject.startsWith(srcInsideProject)) {
            throw new Error("Source file " + sourceFileWithinProject + " not inside src folder " + srcInsideProject);
        }
        return path.join(projectRoot, sourceFileWPToTestFileWP(sourceFileWithinProject));
    };
    return {
        componentDetails: componentDetails,
        fullPathToProjectPath: fullPathToProjectPath,
        isPathTestFile: isPathTestFile,
        isPathWPTestFile: isPathWPTestFile,
        projectRoot: projectRoot,
        sourceFileToModulePath: sourceFileToModulePath,
        sourceFileToTestFile: sourceFileToTestFile,
        sourceFileWPToTestFileWP: sourceFileWPToTestFileWP,
        sourcePathWithinSrc: sourcePathWithinSrc,
        srcInsideProject: srcInsideProject,
        testFileToSourceFile: testFileToSourceFile,
        testFileWPToSourceFileWP: testFileWPToSourceFileWP,
        testInsideProject: testInsideProject,
    };
};
var Formatter = /** @class */ (function () {
    function Formatter(project, ide) {
        this.ide = ide;
        var pkgJson = ide.getPkgJson();
        var eslintDep = R.path(['dependencies', 'eslint'], pkgJson);
        var eslintDevDep = R.path(['devDependencies', 'eslint'], pkgJson);
        if (eslintDep || eslintDevDep) {
            this.linter = new eslint_1.Eslinter(ide.getProjectRoot());
        }
    }
    Formatter.prototype.format = function (text, fileName) {
        try {
            return this.linter ?
                this.linter.format(text, fileName) :
                text;
        }
        catch (e) {
            this.ide.log("Failed to format output: " + e.message);
            return text;
        }
    };
    return Formatter;
}());
exports.Formatter = Formatter;
var FileSystem = /** @class */ (function () {
    function FileSystem() {
    }
    FileSystem.prototype.createComponent = function (component) {
        this.ensureFolderExists(component.folderPath);
        this.ensureFileExists(component.stylesPath);
        this.ensureFileExists(component.componentPath);
    };
    FileSystem.prototype.ensureFolderExists = function (pathToCheck) {
        var dirPath = path.extname(pathToCheck) ? path.dirname(pathToCheck) : pathToCheck;
        if (!fs.existsSync(dirPath)) {
            mkdirp.sync(dirPath);
        }
    };
    FileSystem.prototype.ensureFileExists = function (pathToCheck) {
        this.ensureFolderExists(path.dirname(pathToCheck));
        if (!fs.existsSync(pathToCheck)) {
            fs.closeSync(fs.openSync(pathToCheck, 'w'));
        }
    };
    return FileSystem;
}());
exports.FileSystem = FileSystem;
exports.fancyReactSettings = ["testStructure", "packagePath", "sourcePath", "testPath", "testSuffix"];
__export(require("./actions"));
//# sourceMappingURL=index.js.map